/*
Copyright 2010 WebAtlas
Authors : Mathieu Bastian, Mathieu Jacomy, Julian Bilcke, Patrick J. McSweeney
Website : http://www.gephi.org

This file is part of Gephi.

Gephi is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Gephi is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Gephi.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.gephi.desktop.timeline;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.logging.Logger;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.gephi.project.api.ProjectController;
import org.gephi.timeline.api.TimelineModel;
import org.gephi.timeline.spi.TimelineDrawer;
import org.gephi.project.api.Workspace;
import org.gephi.project.api.WorkspaceListener;
import org.gephi.timeline.api.TimelineAnimator;
import org.gephi.timeline.api.TimelineAnimatorListener;
import org.gephi.timeline.api.TimelineModelListener;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;
import org.openide.util.ImageUtilities;
import org.netbeans.api.settings.ConvertAsProperties;
import org.openide.util.Lookup;

/**
 * Top component which displays something.
 */
@ConvertAsProperties(dtd = "-//org.gephi.desktop.timeline//Timeline//EN",
autostore = false)
public final class TimelineTopComponent
        extends TopComponent
        implements TimelineModelListener,
        TimelineAnimatorListener {

    private static TimelineTopComponent instance;
    /** path to the icon used by the component and its open action */
    static final String ICON_PATH = "org/gephi/desktop/timeline/resources/ui-status-bar.png";
    private static final String PREFERRED_ID = "TimelineTopComponent";
    private TimelineModel model;
    private JPanel drawerPanel;
    private TimelineAnimatorImpl animator;

    public TimelineTopComponent() {
        initComponents();
        setName(NbBundle.getMessage(TimelineTopComponent.class, "CTL_TimelineTopComponent"));
        setToolTipText(NbBundle.getMessage(TimelineTopComponent.class, "HINT_TimelineTopComponent"));
        setIcon(ImageUtilities.loadImage(ICON_PATH, true));
        putClientProperty(TopComponent.PROP_MAXIMIZATION_DISABLED, Boolean.TRUE);


        model = (TimelineModel) Lookup.getDefault().lookup(TimelineModelImpl.class);
        TimelineDrawer drawer = Lookup.getDefault().lookup(TimelineDrawer.class);
        drawerPanel = (JPanel) drawer;
        timelinePanel.add(drawerPanel);

        animator = new TimelineAnimatorImpl();
        animator.addListener(this);

        refreshModel(model);
        //Workspace events
        ProjectController pc = Lookup.getDefault().lookup(ProjectController.class);
        pc.addWorkspaceListener(new WorkspaceListener() {

            public void initialize(Workspace workspace) {
                workspace.add(new TimelineModelImpl());
            }

            public void select(Workspace workspace) {
                TimelineModelImpl m = workspace.getLookup().lookup(TimelineModelImpl.class);
                if (m == null) {
                    m = new TimelineModelImpl();
                    workspace.add(m);
                }
                refreshModel(m);
            }

            public void unselect(Workspace workspace) {
            }

            public void close(Workspace workspace) {
            }

            public void disable() {
                refreshModel(null);
            }
        });

        if (pc.getCurrentWorkspace() != null) {
            TimelineModelImpl m = pc.getCurrentWorkspace().getLookup().lookup(TimelineModelImpl.class);
            if (m == null) {
                m = new TimelineModelImpl();
                pc.getCurrentWorkspace().add(m);
            }
            refreshModel(m);
        } else {
            refreshModel(null);
        }

        //Settings
        settingsButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                //
            }
        });
        settingsButton.setVisible(false);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        timelineToolbar = new javax.swing.JToolBar();
        settingsButton = new javax.swing.JButton();
        timelinePanel = new javax.swing.JPanel();

        setMaximumSize(new java.awt.Dimension(2147483647, 27));
        setMinimumSize(new java.awt.Dimension(128, 16));
        setPreferredSize(new java.awt.Dimension(800, 24));
        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.LINE_AXIS));

        timelineToolbar.setBorder(javax.swing.BorderFactory.createMatteBorder(0, 0, 0, 1, new java.awt.Color(5, 4, 3)));
        timelineToolbar.setFloatable(false);
        timelineToolbar.setOrientation(1);
        timelineToolbar.setRollover(true);
        timelineToolbar.setMaximumSize(new java.awt.Dimension(0, 32767));

        org.openide.awt.Mnemonics.setLocalizedText(settingsButton, org.openide.util.NbBundle.getMessage(TimelineTopComponent.class, "TimelineTopComponent.settingsButton.text")); // NOI18N
        settingsButton.setFocusable(false);
        settingsButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        settingsButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        timelineToolbar.add(settingsButton);

        add(timelineToolbar);

        timelinePanel.setMinimumSize(new java.awt.Dimension(300, 28));
        timelinePanel.setLayout(new java.awt.BorderLayout());
        add(timelinePanel);
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton settingsButton;
    private javax.swing.JPanel timelinePanel;
    private javax.swing.JToolBar timelineToolbar;
    // End of variables declaration//GEN-END:variables

    /**
     * Gets default instance. Do not use directly: reserved for *.settings files only,
     * i.e. deserialization routines; otherwise you could get a non-deserialized instance.
     * To obtain the singleton instance, use {@link #findInstance}.
     */
    public static synchronized TimelineTopComponent getDefault() {
        if (instance == null) {
            instance = new TimelineTopComponent();
        }
        return instance;
    }

    /**
     * Obtain the TimelineTopComponent instance. Never call {@link #getDefault} directly!
     */
    public static synchronized TimelineTopComponent findInstance() {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        if (win == null) {
            Logger.getLogger(TimelineTopComponent.class.getName()).warning(
                    "Cannot find " + PREFERRED_ID + " component. It will not be located properly in the window system.");
            return getDefault();
        }
        if (win instanceof TimelineTopComponent) {
            return (TimelineTopComponent) win;
        }
        Logger.getLogger(TimelineTopComponent.class.getName()).warning(
                "There seem to be multiple components with the '" + PREFERRED_ID
                + "' ID. That is a potential source of errors and unexpected behavior.");
        return getDefault();
    }

    @Override
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_ALWAYS;
    }

    @Override
    public void componentOpened() {
        // TODO add custom code on component opening
    }

    @Override
    public void componentClosed() {
        // TODO add custom code on component closing
    }

    void writeProperties(java.util.Properties p) {
        // better to version settings since initial version as advocated at
        // http://wiki.apidesign.org/wiki/PropertyFiles
        p.setProperty("version", "1.0");
        // TODO store your settings
    }

    Object readProperties(java.util.Properties p) {
        if (instance == null) {
            instance = this;
        }
        instance.readPropertiesImpl(p);
        return instance;
    }

    private void readPropertiesImpl(java.util.Properties p) {
        String version = p.getProperty("version");
        // TODO read your settings according to their version
    }

    @Override
    protected String preferredID() {
        return PREFERRED_ID;
    }

    private void refreshModel(TimelineModel model) {

        if (model == null) {
            refreshEnable(false);
            return;
        }

        if (model != this.model) {
            if (this.model != null) {
                this.model.removeListener(this);
            }
            model.addListener(this);
            this.model = model;
            ((TimelineDrawer) drawerPanel).setModel(model);
        }

        refreshEnable(true);

        // repaint(); // drawerPanel will call the model itself
        // TODO repaint ?
    }

    private void refreshEnable(boolean enable) {
        if (!((JPanel) drawerPanel).isEnabled()) {
            ((JPanel) drawerPanel).setEnabled(enable);
        }
        if (!timelinePanel.isEnabled()) {
            timelinePanel.setEnabled(enable);
        }
        if (!timelineToolbar.isEnabled()) {
            timelineToolbar.setEnabled(enable);
        }
        if (!settingsButton.isEnabled()) {
            settingsButton.setEnabled(enable);
        }
    }

    public void timelineModelChanged(ChangeEvent event) {
        // ..
    }

    public void timelineAnimatorChanged(ChangeEvent event) {
        // check animator value, to update the buttons etc..
    }
}
